/*
   This program builds some more binary trees using our binary tree language.

   This program assumes that you have installed Graphviz, which includes
   the dot.exe program. You can download Graphviz and dot.exe from the
   following URL. Unzip the zip file to your C:\ drive.

     https://graphviz.gitlab.io/_pages/Download/windows/graphviz-2.38.zip
*/

public class BTree_Language_Example_2
{
   public static void main(String[] args)
   {
      String[] btreeStrings = {"( a ( b c d ) ( e f g ) )",
                               "( * ( + x y ) ( - z w ) )",
                               "( / x ( + y z ) )",
                               "( < ( + 23 32 ) 0 )",
                               "( && ( || a ( || b c ) ) ( == ( + x y ) z ) )"
                              };

      for (int i = 0; i < btreeStrings.length; i++)
      {
         System.out.println(i + " =============================================");

         // Build the Tree data structure from its string representation.
         BTree btree = BuildBTree.buildBTree( btreeStrings[i] );

         // Print the BTree using the BTree's toString() method.
         System.out.println("Binary tree expression is " + btree + "\n" );

         // Pretty print the tree in a nice format.
         System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );


         // Create dot and png files from the tree.
         if(true)
         try
         {
            // Create the (empty) dot file.
            String baseName = String.format("Example_3.%02d", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot"));
            // Put dot commands into the dot file
            out.println( BTree2dot.btree2dot(btree) + "\n" );
            out.close();

            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }
}
