/*
   This program demonstrates three ways to
   instantiate the same BTree data structure.
      Using nested constructors.
      By adding sub trees to a tree.
      Using the binary tree language.

               apple
               /   \
              /     \
             ()     pear
                    /  \
                   /    \
                plum    ()

   This program assumes that you have installed Graphviz, which includes
   the dot.exe program. You can download Graphviz and dot.exe from the
   following URL. Unzip the zip file to your C:\ drive.

     https://graphviz.gitlab.io/_pages/Download/windows/graphviz-2.38.zip
*/

public class BTree_Language_Example_3
{
   public static void main(String[] args)
   {
      // Build the BTree data structure using nested constructors.
      BTree btree1 = new BTree("apple", null, new BTree("pear", new BTree("plum"), null));

      // Build the BTree data structure by adding sub trees.
      BTree btree2 = new BTree("apple");
      btree2.addRightTree(new BTree("pear"));
      btree2.getRightTree().addLeftTree(new BTree("plum"));

      // Build the BTree data structure from its string representation.
      BTree btree3 = BuildBTree.buildBTree("( apple () ( pear plum () ) )");

      // Print each BTree using the BTree's toString() method.
      System.out.println("Binary tree expression is " + btree1);
      System.out.println("Binary tree expression is " + btree2);
      System.out.println("Binary tree expression is " + btree3);

      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing a string
      // from the binary tree language so a "pretty-printer" is
      // sometimes called an "unparser". This is also sometimes
      // referred to as "serializing" the binary tree data structure
      // because the resulting string is a linear data structure.
      System.out.println( PrettyPrinter.prettyPrint( btree1 ) + "\n" );
      System.out.println( PrettyPrinter.prettyPrint( btree2 ) + "\n" );
      System.out.println( PrettyPrinter.prettyPrint( btree3 ) + "\n" );
   }
}
